@@ -4,17 +4,36 @@
 # 2013-08-19: removed loading of stats package. J. Fox
 # 2016-09-30: added list, data.frame, and matrix methods, suggestion of Michael Friendly. J. Fox
 # 2016-10-01: tweaked data.frame and list methods. J. Fox
+# 2017-01-11: consolidate id argument
+# 2017-10-03: add col and cex to id argument
+
 Boxplot <- function(y, ...){
+  arg.list <- list(...)
+  if (!is.null(arg.list$horizontal) && isTRUE(arg.list$horizontal))
+    stop("Boxplot does not support horizontal=TRUE")
 	UseMethod("Boxplot")
 }
 
-Boxplot.default <- function (y, g, labels, id.method = c("y", "identify", "none"), 
-    id.n = 10, xlab, ylab, ...) {
-    id.method <- match.arg(id.method)
+Boxplot.default <- function(y, g, id=TRUE, xlab, ylab, ...){
+    if (isFALSE(id)) {
+        id.method="none"
+        labels <- NULL
+        id.n <- 0
+        id.cex <- NULL
+        id.col <- NULL
+    }
+    else{
+        id <- applyDefaults(id, defaults=list(method="y", n=10, location="lr", cex=1,  col=carPalette()[1]), type="id")
+        id.method <- match.arg(id$method, c("y", "identify", "none"))
+        id.n <- id$n
+        id.location <- id$location
+        labels <- if (is.null(id$labels)) seq(along = y)
+            else id$labels
+        id.cex <- id$cex
+        id.col <- id$col
+    }
     if (missing(ylab)) 
         ylab <- deparse(substitute(y))
-    if (missing(labels)) 
-        labels <- seq(along = y)
     pars <- list(...)
     if (missing(g)) {
         valid <- complete.cases(y, labels)
@@ -24,7 +43,7 @@ Boxplot.default <- function (y, g, labels, id.method = c("y", "identify", "none"
         if (id.method == "none" | id.n == 0) 
             return(invisible(NULL))
         else if (id.method == "identify") {
-            res <- identify(rep(1, length(y)), y, labels)
+            res <- identify(rep(1, length(y)), y, labels, cex=id.cex, col=id.col)
             return(if (length(res) == 0) invisible(NULL) else labels[res])
         }
         else if (length(b$out) > 0) {
@@ -48,8 +67,11 @@ Boxplot.default <- function (y, g, labels, id.method = c("y", "identify", "none"
                 labs.high <- labs.high[ord.high]
             }
             labs <- c(labs.low, labs.high)
-            at <- if(!is.null(pars$at)) pars$at else 1			#@@@
-            text(at, c(y.low, y.high), labs, pos = 2)			#@@@
+            at <- if(!is.null(pars$at)) pars$at else 1		
+            if (id.location == "lr") text(at, c(y.low, y.high), labs, pos = 2, xpd=TRUE, cex=id.cex, col=id.col)
+            else maptools::pointLabel(c(at, at), c(y.low, y.high, y.low, y.high), 
+                                      c(paste0(" ", labs, " "), rep(" ", length(labs))), 
+                                      xpd=TRUE, col=id.col, cex=id.cex)
             return(if (length(labs) == 0) invisible(NULL) else labs)
         }
         else return(invisible(NULL))
@@ -104,8 +126,11 @@ Boxplot.default <- function (y, g, labels, id.method = c("y", "identify", "none"
                         4
                     else 2
                     at <- if(!is.null(pars$at)) pars$at[group] else group
-                    text(at, c(y.low, y.high), c(labs.low, labs.high),
-                        pos = pos)
+                    labs <- c(labs.low, labs.high)
+                    if (id.location == "lr") text(at, c(y.low, y.high), labs, pos = pos, xpd=TRUE, col=id.col, cex=id.cex)
+                    else maptools::pointLabel(c(at, at), c(y.low, y.high, y.low, y.high), 
+                                                   c(paste0(" ", labs, " "), rep(" ", length(labs))), 
+                                                   xpd=TRUE, col=id.col, cex=id.cex)
                     identified <- c(identified, c(labs.low, labs.high))
                 }
             }
@@ -115,18 +140,33 @@ Boxplot.default <- function (y, g, labels, id.method = c("y", "identify", "none"
 }
 
 
-Boxplot.formula <- function(formula, data=NULL, subset, na.action=NULL, labels., 
-  id.method=c("y", "identify", "none"), xlab, ylab, ...){
+Boxplot.formula <- function(formula, data=NULL, subset, na.action=NULL, id=TRUE, xlab, ylab, ...){
 	# much of this function adapted from graphics:boxplot.formula
-	id.method <- match.arg(id.method)
+    if (isFALSE(id)) {
+        id.method="none"
+        id.n <- 0
+        labels <- NULL
+        id.location <- "y"
+        id.col <- NULL
+        id.cex <- NULL
+    }
+    else{
+        id <- applyDefaults(id, defaults=list(method="y", n=10, location="lr", cex=1,  col=carPalette()[1]), type="id")
+        id.method <- match.arg(id$method, c("y", "identify", "none"))
+        id.n <- id$n
+        id.location <- id$location
+        labels <- id$labels
+        id.col <- id$col
+        id.cex <- id$cex
+    }
 	m <- match.call(expand.dots = FALSE)
 	if (is.matrix(eval(m$data, parent.frame()))) 
 		m$data <- as.data.frame(data)
-	m$xlab <- m$ylab <- m$id.method <- m$... <- NULL
+	m$xlab <- m$ylab <- m$id <- m$... <- NULL
 	m$na.action <- na.action
 	m[[1]] <- as.name("model.frame")
 	mf <- eval(m, parent.frame())
-	if (missing(labels.)) mf$"(labels.)" <- rownames(mf)
+	mf$"(labels.)" <- if (is.null(labels)) rownames(mf) else labels
 	lab.var <- which(names(mf) == "(labels.)")
 	if (length(formula) == 3){
 		response <- attr(attr(mf, "terms"), "response")
@@ -135,12 +175,14 @@ Boxplot.formula <- function(formula, data=NULL, subset, na.action=NULL, labels.,
         x <- mf[, -c(response, lab.var)]
 		if (is.data.frame(x)) x <- do.call("interaction", as.list(x))
         if (length(xlab) > 1) xlab <- paste(xlab, collapse="*")
-		Boxplot(mf[[response]], x, labels=mf[[lab.var]], 
-            xlab=xlab, ylab=ylab, id.method=id.method, ...)
+		Boxplot(mf[[response]], x, id=list(method=id.method, labels=mf[[lab.var]], 
+		                                   n=id.n, location=id.location, col=id.col, cex=id.cex), 
+            xlab=xlab, ylab=ylab, ...)
 	}
 	else if (length(formula) == 2){
 		if (missing(ylab)) ylab <- names(mf)[-lab.var]
-		Boxplot(mf[, -lab.var], labels=mf[[lab.var]], ylab=ylab, id.method=id.method, ...)
+		Boxplot(mf[, -lab.var], id=list(method=id.method, 
+		                                labels=mf[[lab.var]], n=id.n, location=id.location, col=id.col, cex=id.cex), ylab=ylab, ...)
 	}
 	else stop("improper Boxplot formula")   
 }
@@ -152,9 +194,25 @@ Boxplot.list <- function(y, xlab="", ylab="", ...){
   Boxplot(y, g, xlab=xlab, ylab=ylab, ...)
 }
 
-Boxplot.data.frame <-  function(y, labels=rownames(y), ...){
-  labels <- rep(labels, ncol(y))
-  Boxplot(as.list(y), labels=labels, ...)
+Boxplot.data.frame <-  function(y, id=TRUE, ...){
+    if (isFALSE(id)) {
+        id.method="none"
+        id.n <- 0
+        labels <- NULL
+        id.location <- "y"
+        id.col <- NULL
+        id.cex <- NULL
+    }
+    else{
+        id <- applyDefaults(id, defaults=list(method="y", n=10, location="lr", labels=rownames(y), cex=1,  col=carPalette()[1]), type="id")
+        id.method <- match.arg(id$method, c("y", "identify", "none"))
+        id.n <- id$n
+        id.location <- id$location
+        labels <- rep(id$labels, ncol(y))
+        id.col <- id$col
+        id.cex <- id$cex
+    }
+  Boxplot(as.list(y), id=list(method=id.method, n=id.n, location=id.location, labels=labels, cex=id.cex, col=id.col), ...)
 }
 
 Boxplot.matrix <- function(y, ...){
